'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babylon = require('babylon');

var babylon = _interopRequireWildcard(_babylon);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var typeInfoRegex = /^:([a-z])(\((.+)\))?/;
var paramRegex = /\$\{(\d+)\}/g;

var buildKey = function buildKey(literals) {
    var stripType = function stripType(s) {
        return s.replace(typeInfoRegex, '');
    };
    var lastPartialKey = stripType(literals[literals.length - 1]);
    var prependPartialKey = function prependPartialKey(memo, curr, i) {
        return stripType(curr) + '${' + i + '}' + memo;
    };
    return literals.slice(0, -1).reduceRight(prependPartialKey, lastPartialKey).replace(/\r\n/, '\n');
};

var updateTemplateLiterals = {
    TaggedTemplateExpression: function TaggedTemplateExpression(path, _ref) {
        var translations = _ref.translations;
        var filename = _ref.filename;
        var groupName = _ref.groupName;

        var node = path.node;
        if (!node.translated && node.tag.name === 'i18n' || node.tag.callee && (node.tag.callee.name === 'i18n' || node.tag.callee.property && node.tag.callee.property.name === 'i18n') || node.tag.property && node.tag.property.name === 'i18n') {
            var quasi = node.quasi || node.tag.callee.quasi || node.tag.callee.property.quasi;

            var quasis = quasi.quasis.map(function (n) {
                return n.value.raw;
            });
            var ext = quasis.map(function (q) {
                var match = q.match(typeInfoRegex);
                if (match && match.length > 1) {
                    return match[0];
                }
                return '';
            });
            var key = buildKey(quasis);

            if (node.tag.arguments && node.tag.arguments.length) {
                groupName = path.node.tag.arguments[0].value;
            }

            var translation = void 0;
            if (groupName) {
                var group = translations[groupName];
                if (group && group instanceof Object) translation = group[key];
            } else if (filename) {
                var _group = translations[filename];
                if (_group && _group instanceof Object) translation = _group[key];
            }

            if (!translation) {
                translation = translations[key];
            }

            if (translation) {
                var indices = [];
                var paramMatch = void 0;
                while ((paramMatch = paramRegex.exec(translation)) && paramMatch.length == 2) {
                    indices.push(Number.parseInt(paramMatch[1]));
                }
                var literals = translation.split(/\$\{\d+\}/g);
                var _quasis = [];
                var expressions = [];
                var nextPrefix = '';
                for (var i = 0; i < literals.length; i++) {
                    _quasis.push(t.templateElement({ raw: nextPrefix + literals[i], cooked: nextPrefix + literals[i] }, i === literals.length - 1));
                    if (indices.length > i) {
                        var paramIndex = indices[i];
                        if (ext.length > paramIndex + 1) {
                            nextPrefix = ext[paramIndex + 1];
                        } else {
                            nextPrefix = '';
                        }
                        if (path.node.quasi.expressions.length > paramIndex) {
                            var expression = path.node.quasi.expressions[paramIndex];
                            expressions.push(expression);
                        }
                    }
                }
                var literal = t.templateLiteral(_quasis, expressions);
                var taggedTemplate = t.taggedTemplateExpression(path.node.tag, literal);
                path.replaceWith(taggedTemplate);
                taggedTemplate.translated = true;
            }
        }
    }
};

var traverseClassDeclarations = {
    ClassDeclaration: function ClassDeclaration(path, _ref2) {
        var translations = _ref2.translations;
        var filename = _ref2.filename;
        var groups = _ref2.groups;

        var node = path.node;
        if (node.decorators && node.decorators.length) {
            var groupNames = node.decorators.map(function (d) {
                return d.expression;
            }).filter(function (e) {
                return e.callee && e.callee.name === 'i18nGroup' && e.arguments && e.arguments.length;
            }).map(function (d) {
                return d.arguments.map(function (a) {
                    return a.value;
                });
            }).reduce(function (p, n) {
                return p.concat(n);
            }, []);
            var groupName = groupNames.length ? groupNames[0] : null;
            path.traverse(updateTemplateLiterals, { translations: translations, filename: filename, groupName: groupName });
        } else {
            path.traverse(updateTemplateLiterals, { translations: translations, filename: filename, groupName: groups[node.id.name] });
        }
    }
};

var traverseExportDeclarations = {
    CallExpression: function CallExpression(path, _ref3) {
        var groups = _ref3.groups;

        var node = path.node;
        if (node.callee && node.callee.type === 'CallExpression' && node.callee.callee && node.callee.callee.type === 'Identifier' && node.callee.callee.name === 'i18nGroup' && node.callee.arguments && node.callee.arguments.length && node.arguments && node.arguments.length) {
            groups[node.arguments[0].name] = node.callee.arguments[0].value;
        }
    }
};

var babelPlugin = function babelPlugin() {
    return {
        visitor: {
            Program: function Program(p, _ref4) {
                var file = _ref4.file;
                var opts = _ref4.opts;

                if (opts['config']) {
                    // TODO Clean up this mess
                    var code = 'let configs = ' + JSON.stringify(opts['config']) + ';';
                    var objectExpression = babylon.parse(code).program.body[0].declarations[0].init;
                    var ast = t.expressionStatement(t.callExpression(t.identifier('i18nConfig'), [objectExpression]));
                    p.unshiftContainer('body', ast);
                }

                if (opts['globalImport'] || opts['config']) {
                    var newImport = t.importDeclaration([t.importDefaultSpecifier(t.identifier('i18n')), t.importSpecifier(t.identifier('i18nConfig'), t.identifier('i18nConfig')), t.importSpecifier(t.identifier('i18nGroup'), t.identifier('i18nGroup'))], t.stringLiteral('es2015-i18n-tag'));

                    p.unshiftContainer('body', newImport);
                }

                var filename = '';
                if (opts.groupDir) {
                    filename = _path2.default.relative(_path2.default.resolve(process.cwd(), opts.groupDir), file.opts.filename).replace(/\\/g, '/');
                    var _code = 'const __translationGroup = ' + JSON.stringify(filename) + ';';
                    var _objectExpression = babylon.parse(_code).program.body[0];
                    p.unshiftContainer('body', _objectExpression);
                }

                var translations = {};
                if (opts.translation) {
                    try {
                        var translationsFile = _path2.default.resolve(process.cwd(), opts.translation);
                        console.log('Reading: ' + translationsFile + ' ...');
                        translations = JSON.parse(_fs2.default.readFileSync(translationsFile, 'utf-8'));
                        if (opts.config && opts.config.translations) {
                            translations = Object.assign(translations, opts.config.translations);
                        }
                        console.log('Successfully imported translations.');
                    } catch (err) {
                        console.warn(err.message);
                    }
                }
                var groups = {};
                p.traverse(traverseExportDeclarations, { groups: groups }); // find all i18nGroup calls
                p.traverse(traverseClassDeclarations, { translations: translations, filename: filename, groups: groups }); // traverse classes first to get group decorators
                p.traverse(updateTemplateLiterals, { translations: translations, filename: filename });
            }
        }
    };
};

exports.default = babelPlugin;